@@ -87,6 +87,7 @@ void setThemePalette_Color(enum iColorTheme theme) {
             copy_(uiBackgroundFramelessHover_ColorId, accentLo);
             set_Color(uiBackgroundSidebar_ColorId,
                       mix_Color(get_Color(black_ColorId), get_Color(gray25_ColorId), 0.55f));
+            copy_(uiBackgroundMenu_ColorId, uiBackground_ColorId);
             copy_(uiText_ColorId, gray75_ColorId);
             copy_(uiTextPressed_ColorId, black_ColorId);
             copy_(uiTextStrong_ColorId, white_ColorId);
@@ -136,6 +137,7 @@ void setThemePalette_Color(enum iColorTheme theme) {
             copy_(uiBackgroundFramelessHover_ColorId, accentLo);
             set_Color(uiBackgroundSidebar_ColorId,
                       mix_Color(get_Color(black_ColorId), get_Color(gray25_ColorId), 0.75f));
+            copy_(uiBackgroundMenu_ColorId, uiBackground_ColorId);
             copy_(uiText_ColorId, gray75_ColorId);
             copy_(uiTextPressed_ColorId, black_ColorId);
             copy_(uiTextStrong_ColorId, white_ColorId);
@@ -184,7 +186,8 @@ void setThemePalette_Color(enum iColorTheme theme) {
             copy_(uiBackgroundPressed_ColorId, altAccentHi);
             copy_(uiBackgroundFramelessHover_ColorId, accentHi);
             set_Color(uiBackgroundSidebar_ColorId,
-                      mix_Color(get_Color(white_ColorId), get_Color(gray75_ColorId), 0.5f));
+                      mix_Color(get_Color(white_ColorId), get_Color(gray75_ColorId), 0.67f));
+            copy_(uiBackgroundMenu_ColorId, white_ColorId);
             copy_(uiText_ColorId, black_ColorId);
             copy_(uiTextStrong_ColorId, black_ColorId);
             copy_(uiTextDim_ColorId, gray25_ColorId);
@@ -234,10 +237,12 @@ void setThemePalette_Color(enum iColorTheme theme) {
             copy_(uiBackgroundFramelessHover_ColorId, accentHi);
             set_Color(uiBackgroundSidebar_ColorId,
                       mix_Color(get_Color(white_ColorId), get_Color(gray75_ColorId), 0.5f));
+            copy_(uiBackgroundMenu_ColorId, uiBackground_ColorId);
             set_Color(uiText_ColorId,
                       mix_Color(get_Color(black_ColorId), get_Color(gray25_ColorId), 0.5f));
             copy_(uiTextPressed_ColorId, black_ColorId);
-            copy_(uiTextDisabled_ColorId, gray75_ColorId);
+            set_Color(uiTextDisabled_ColorId,
+                      mix_Color(get_Color(gray75_ColorId), get_Color(gray50_ColorId), 0.5f));
             copy_(uiTextStrong_ColorId, black_ColorId);
             copy_(uiTextDim_ColorId, gray25_ColorId);
             copy_(uiTextSelected_ColorId, black_ColorId);
@@ -290,8 +295,17 @@ void setThemePalette_Color(enum iColorTheme theme) {
                                     theme == pureBlack_ColorTheme ? -1
                                     : theme == dark_ColorTheme    ? -0.04
                                                                   : -0.075));
+    set_Color(uiTextShortcut_ColorId, mix_Color(get_Color(uiTextShortcut_ColorId),
+                                                get_Color(uiBackground_ColorId),
+                                                0.4f));
     palette_[uiMarked_ColorId].a = 128;
     palette_[uiMatching_ColorId].a = 128;
+    if (deviceType_App() == phone_AppDeviceType) {
+        copy_(uiInputBackground_ColorId, uiBackgroundSidebar_ColorId);
+        copy_(uiInputFrame_ColorId, uiBackgroundSidebar_ColorId);
+        copy_(uiInputFrameFocused_ColorId, uiBackgroundSidebar_ColorId);
+        copy_(uiInputBackgroundFocused_ColorId, uiBackgroundSidebar_ColorId);
+    }
 }
 
 iColor get_Color(int color) {
