@@ -72,9 +72,12 @@ struct Impl_Window {
     uint32_t      frameTime;
     double        presentTime;
     SDL_Texture * appIcon;
+    SDL_Texture * borderShadow;
     SDL_Cursor *  cursors[SDL_NUM_SYSTEM_CURSORS];
     SDL_Cursor *  pendingCursor;
     int           loadAnimTimer;
+    iAnim         rootOffset;
+    int           keyboardHeight; /* mobile software keyboards */
 };
 
 iBool       processEvent_Window     (iWindow *, const SDL_Event *);
@@ -86,9 +89,16 @@ void        setUiScale_Window       (iWindow *, float uiScale);
 void        setFreezeDraw_Window    (iWindow *, iBool freezeDraw);
 void        setCursor_Window        (iWindow *, int cursor);
 void        setSnap_Window          (iWindow *, int snapMode);
+void        setKeyboardHeight_Window(iWindow *, int height);
+void        dismissPortraitPhoneSidebars_Window (iWindow *);
+void        showToolbars_Window     (iWindow *, iBool show);
+iBool       postContextClick_Window (iWindow *, const SDL_MouseButtonEvent *);
 
 uint32_t    id_Window               (const iWindow *);
 iInt2       rootSize_Window         (const iWindow *);
+iRect       safeRootRect_Window     (const iWindow *);
+iInt2       visibleRootSize_Window  (const iWindow *); /* may be obstructed by software keyboard */
+iInt2       maxTextureSize_Window   (const iWindow *);
 float       uiScale_Window          (const iWindow *);
 iInt2       coord_Window            (const iWindow *, int x, int y);
 iInt2       mouseCoord_Window       (const iWindow *);
@@ -96,6 +106,7 @@ uint32_t    frameTime_Window        (const iWindow *);
 SDL_Renderer *renderer_Window       (const iWindow *);
 int         snap_Window             (const iWindow *);
 iBool       isFullscreen_Window     (const iWindow *);
+iBool       isNarrow_Window         (const iWindow *);
 
 iWindow *   get_Window              (void);
 
